function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

import { Stream, of, zip } from 'most';
import { equals } from 'ramda';

import Logger from 'mtproto-logger';
var log = Logger`net-request`;

import { API, NET } from '../action';
import { NetMessage } from '../../service/networker/net-message';
import { apiMessage, encryptApiBytes, mtMessage } from '../../service/chain/encrypted-message';
import Config from '../../config-provider';
import NetworkerThread from '../../service/networker/index';
import { Serialization } from '../../tl/index';
import { httpClient } from '../../http';
import { homeDc, uid, whenActive } from '../signal';
import { faucet } from '../../pull-stream';
import jsonError from '../../util/json-error';

function wait(val) {
  return of(val).awaitPromises();
}

var makeApiBytes = ({ message, thread }) => apiMessage({
  ctx: new Serialization({ startMaxLength: message.body.length + 64 }, thread.uid).writer,
  serverSalt: thread.serverSalt,
  sessionID: thread.sessionID,
  message
});

var encryptedBytes = opts => encryptApiBytes({
  bytes: makeApiBytes(opts),
  authKey: opts.thread.authKeyUint8
});

export var onNewRequest = action => action.thru(e => API.NEW_REQUEST.stream(e)).thru(whenActive).combine((data, homeDc) => Object.assign({}, data, { homeDc }), homeDc).combine((data, uid) => Object.assign({}, data, { uid }), uid).skipRepeatsWith((old, fresh) => equals(old.payload, fresh.payload)).map(data => data).delay(50).tap(val => val.payload.netReq.invoke()).filter(() => false);

var netRequest = action => action.thru(e => NET.SEND.stream(e)).thru(whenActive).map(({ payload }) => payload).chain(opts => zip((opts, data) => Object.assign({}, opts, data), of(opts), wait(encryptedBytes(opts)))).map((_ref) => {
  var { options } = _ref,
      rest = _objectWithoutProperties(_ref, ['options']);

  return Object.assign({}, rest, {
    options: Object.assign({
      responseType: 'arraybuffer' }, options) });
}).map((_ref2) => {
  var { encryptedBytes, thread, msgKey } = _ref2,
      rest = _objectWithoutProperties(_ref2, ['encryptedBytes', 'thread', 'msgKey']);

  var request = new Serialization({
    startMaxLength: encryptedBytes.byteLength + 256
  }, thread.uid).writer;

  var mtBytes = mtMessage({
    ctx: request,
    authKeyID: thread.authKeyID,
    msgKey,
    encryptedBytes
  });
  var url = Config.dcMap(thread.uid, thread.dcID);
  return Object.assign({ thread, mtBytes, url }, rest);
}).map((() => {
  var _ref3 = _asyncToGenerator(function* ({ options, mtBytes, message, thread, noResponseMsgs, url }) {
    return {
      message,
      thread,
      noResponseMsgs,
      result: yield httpClient.post(url, mtBytes, options)
    };
  });

  return function (_x) {
    return _ref3.apply(this, arguments);
  };
})()).chain(val => wait(val)).map(NET.RECEIVE_RESPONSE).recoverWith(err => of(NET.NETWORK_ERROR(jsonError(err))).delay(15));

export default netRequest;
//# sourceMappingURL=net-request.js.map