'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.onNewRequest = undefined;

var _most = require('most');

var _ramda = require('ramda');

var _mtprotoLogger = require('mtproto-logger');

var _mtprotoLogger2 = _interopRequireDefault(_mtprotoLogger);

var _action = require('../action');

var _netMessage = require('../../service/networker/net-message');

var _encryptedMessage = require('../../service/chain/encrypted-message');

var _configProvider = require('../../config-provider');

var _configProvider2 = _interopRequireDefault(_configProvider);

var _index = require('../../service/networker/index');

var _index2 = _interopRequireDefault(_index);

var _index3 = require('../../tl/index');

var _http = require('../../http');

var _signal = require('../signal');

var _pullStream = require('../../pull-stream');

var _jsonError = require('../../util/json-error');

var _jsonError2 = _interopRequireDefault(_jsonError);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var log = _mtprotoLogger2.default`net-request`;

function wait(val) {
  return (0, _most.of)(val).awaitPromises();
}

var makeApiBytes = ({ message, thread }) => (0, _encryptedMessage.apiMessage)({
  ctx: new _index3.Serialization({ startMaxLength: message.body.length + 64 }, thread.uid).writer,
  serverSalt: thread.serverSalt,
  sessionID: thread.sessionID,
  message
});

var encryptedBytes = opts => (0, _encryptedMessage.encryptApiBytes)({
  bytes: makeApiBytes(opts),
  authKey: opts.thread.authKeyUint8
});

var onNewRequest = exports.onNewRequest = action => action.thru(e => _action.API.NEW_REQUEST.stream(e)).thru(_signal.whenActive).combine((data, homeDc) => Object.assign({}, data, { homeDc }), _signal.homeDc).combine((data, uid) => Object.assign({}, data, { uid }), _signal.uid).skipRepeatsWith((old, fresh) => (0, _ramda.equals)(old.payload, fresh.payload)).map(data => data).delay(50).tap(val => val.payload.netReq.invoke()).filter(() => false);

var netRequest = action => action.thru(e => _action.NET.SEND.stream(e)).thru(_signal.whenActive).map(({ payload }) => payload).chain(opts => (0, _most.zip)((opts, data) => Object.assign({}, opts, data), (0, _most.of)(opts), wait(encryptedBytes(opts)))).map((_ref) => {
  var { options } = _ref,
      rest = _objectWithoutProperties(_ref, ['options']);

  return Object.assign({}, rest, {
    options: Object.assign({
      responseType: 'arraybuffer' }, options) });
}).map((_ref2) => {
  var { encryptedBytes, thread, msgKey } = _ref2,
      rest = _objectWithoutProperties(_ref2, ['encryptedBytes', 'thread', 'msgKey']);

  var request = new _index3.Serialization({
    startMaxLength: encryptedBytes.byteLength + 256
  }, thread.uid).writer;

  var mtBytes = (0, _encryptedMessage.mtMessage)({
    ctx: request,
    authKeyID: thread.authKeyID,
    msgKey,
    encryptedBytes
  });
  var url = _configProvider2.default.dcMap(thread.uid, thread.dcID);
  return Object.assign({ thread, mtBytes, url }, rest);
}).map((() => {
  var _ref3 = _asyncToGenerator(function* ({ options, mtBytes, message, thread, noResponseMsgs, url }) {
    return {
      message,
      thread,
      noResponseMsgs,
      result: yield _http.httpClient.post(url, mtBytes, options)
    };
  });

  return function (_x) {
    return _ref3.apply(this, arguments);
  };
})()).chain(val => wait(val)).map(_action.NET.RECEIVE_RESPONSE).recoverWith(err => (0, _most.of)(_action.NET.NETWORK_ERROR((0, _jsonError2.default)(err))).delay(15));

exports.default = netRequest;
//# sourceMappingURL=net-request.js.map